/*
 * flash.c - flash handler
 *
 * Copyright (C) 2004-2006 Atmark Techno, Inc.  All Rights Reserved.
 *
 */

#include <xparameters.h>

#ifndef XPAR_SPI_FLASH_BASEADDR

#include "memmap.h"

#define FLASH_SECTOR_SIZE      0x00010000

typedef unsigned short __u16;

#define ___swab16(x)                                                    \
        ({                                                              \
                __u16 __x = (x);                                        \
                ((__u16)(                                               \
                         (((__u16)(__x) & (__u16)0x00ffU) << 8) |       \
                         (((__u16)(__x) & (__u16)0xff00U) >> 8) ));     \
        })

#define flash_read(addr)                                                \
        ({ unsigned short __v = (*(volatile unsigned short *) (addr)); __v; })
#define flash_write_cmd(addr, b)                                        \
        (void)((*(volatile unsigned short *) (addr)) = (___swab16(b)))
#define flash_write(addr, b)                                    \
        (void)((*(volatile unsigned short *) (addr)) = ((b)))
#define flash_read_cmd(base, offset)                                    \
        ({ unsigned short __v = (*(volatile unsigned short *) (base + (offset<<1))); ___swab16(__v); })

int flash_get_size(void)
{
        int ret;

        flash_write_cmd(FLASH_START, 0xf0);
        flash_write_cmd(FLASH_START + 0xaa, 0x98);
        if (flash_read_cmd(FLASH_START, 0x10) == 'Q' &&
            flash_read_cmd(FLASH_START, 0x11) == 'R' &&
            flash_read_cmd(FLASH_START, 0x12) == 'Y') {
                ret = flash_read_cmd(FLASH_START, 0x27);
        } else {
                ret = -1;
        }
        flash_write_cmd(FLASH_START, 0xf0);

        return ret;
}

void flash_copy_to_dram(unsigned int offset)
{
        int i;
        volatile unsigned short *dst = (unsigned short *)BBOOT_LOAD_ADDRESS;

        for (i = 0; i < BOOTLOADER_SIZE; i += 2) {
                *dst++ = flash_read(FLASH_START + offset + i);
        }
        return;
}

void flash_program(unsigned long addr, unsigned short val)
{
        flash_write_cmd(FLASH_START + 0xaaa, 0xaa);
        flash_write_cmd(FLASH_START + 0x554, 0x55);
        flash_write_cmd(FLASH_START + 0xaaa, 0xa0);
        flash_write(addr, val);
        while (flash_read(addr) != val);
}

void flash_erase(unsigned int offset)
{
        unsigned long i;

        flash_write_cmd(FLASH_START + 0xaaa, 0xaa);
        flash_write_cmd(FLASH_START + 0x554, 0x55);
        flash_write_cmd(FLASH_START + 0xaaa, 0x80);
        flash_write_cmd(FLASH_START + 0xaaa, 0xaa);
        flash_write_cmd(FLASH_START + 0x554, 0x55);

        for (i = 0; i < BOOTLOADER_SIZE; i += FLASH_SECTOR_SIZE) {
                flash_write_cmd(FLASH_START + offset + i, 0x30);
        }

        while (flash_read(FLASH_START + offset + BOOTLOADER_SIZE - FLASH_SECTOR_SIZE) != 0xffff);

        return;
}

#endif /* !XPAR_SPI_FLASH_BASEADDR */
